//+------------------------------------------------------------------+
//|                                                      SOMNode.mqh |
//|                                            Copyright 2015, Urain |
//|                            https://login.mql5.com/ru/users/Urain |
//+------------------------------------------------------------------+
#property copyright "Copyright 2015, Urain"
#property link      "https://login.mql5.com/ru/users/Urain"
//+------------------------------------------------------------------+
//| #include <SOMNode.mqh>                                           |
//+------------------------------------------------------------------+
//+------------------------------------------------------------------+
//|   SOM-                                              |
//+------------------------------------------------------------------+
class CSOMNode
  {
protected:
   //---   
   int               m_x1;
   int               m_y1;
   int               m_x2;
   int               m_y2;
   //---   
   double            m_x;
   double            m_y;
   //---  
   double            m_weights[];
   //---  
   int               m_dimension;
public:
   //---  
                     CSOMNode();
   //---  
                    ~CSOMNode();
   //---    
   void              InitNode(int x1,int y1,int x2,int y2);
   void              InitNode(int x1,int y1,int x2,int y2,const double &min_values[],const double &max_values[]);
   //---     
   double            X()  const { return(m_x);}
   double            Y()  const { return(m_y);}
   int               nX()  const { return((int)NormalizeDouble(m_x,0)/m_dimension);}
   int               nY()  const { return((int)NormalizeDouble(m_y,0)/m_dimension);}
   CSOMNode         *GetObjPointer(){return(GetPointer(this));};
   //---    
   void              GetCoordinates(int &x1,int &y1,int &x2,int &y2);
   //---     weight_index
   double            GetWeight(int weight_index);
   //---     weight_index   
   double            GetWeightR(int weight_index){ return(m_weights[weight_index]); };
   //---     weight_index   
   void              SetWeightR(int weight_index,double volume){ m_weights[weight_index]=volume; };
   //---       
   double            CalculateDistance(const double &vector[]);
   double            CalculateDistance(const double &vector[],int dimension);
   double            CalculateDistance(const double &vector[],const bool &byte_map[]);
   //---    
   void              AdjustWeights(double &vector[],double learning_rate,double influence);
   //---    
   void              SetDimension(int dimension);
   //---    
   int               GetDimension();
   //---    
   int               SaveNode(double &save_node[]);
   //---    
   void              DownloadNode(const double &save_node[]);
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void  CSOMNode::DownloadNode(const double &downl_node[])
  {
   m_dimension=ArraySize(downl_node)-6;
   ArrayResize(m_weights,m_dimension);
   m_x1=(int)NormalizeDouble(downl_node[0],0);
   m_y1=(int)NormalizeDouble(downl_node[1],0);
   m_x2=(int)NormalizeDouble(downl_node[2],0);
   m_y2=(int)NormalizeDouble(downl_node[3],0);
   m_x=downl_node[4];
   m_y=downl_node[5];
   for(int i=6,j=0;i<6+m_dimension;i++,j++)
      m_weights[j]=downl_node[i];
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int  CSOMNode::SaveNode(double &save_node[])
  {
   int size=ArrayResize(save_node,6+m_dimension);
   save_node[0]=m_x1;
   save_node[1]=m_y1;
   save_node[2]=m_x2;
   save_node[3]=m_y2;
   save_node[4]=m_x;
   save_node[5]=m_y;
   for(int i=6,j=0;i<6+m_dimension;i++,j++)
      save_node[i]=m_weights[j];
   return(size);
  };
//+------------------------------------------------------------------+
//|                                                 |
//+------------------------------------------------------------------+
void CSOMNode::CSOMNode()
  {
   m_x1=0;
   m_y1=0;
   m_x2=0;
   m_y2=0;
   m_x=0.0;
   m_y=0.0;
  };
//+------------------------------------------------------------------+
//|                                                  |
//+------------------------------------------------------------------+
void CSOMNode::~CSOMNode()
  {
   ArrayResize(m_weights,0);
  };
//+------------------------------------------------------------------+
//|                                       |
//+------------------------------------------------------------------+
void CSOMNode::InitNode(int x1,int y1,int x2,int y2)
  {
//---   
   m_x1=x1;
   m_y1=y1;
   m_x2=x2;
   m_y2=y2;
//---    
   m_x=x1+MathAbs((x2-x1))/2;
   m_y=y1+MathAbs((y2-y1))/2;
//---      
   ArrayResize(m_weights,m_dimension);
//---    ;
   for(int i=0; i<m_dimension; i++) {m_weights[i]=255.0*rand()/32768;}
  };
//+------------------------------------------------------------------+
//|                        |
//+------------------------------------------------------------------+
void CSOMNode::InitNode(int x1,int y1,int x2,int y2,const double &min_values[],const double &max_values[])
  {
//---   
   m_x1=x1;
   m_y1=y1;
   m_x2=x2;
   m_y2=y2;
//---    
   m_x=x1+MathAbs((x2-x1))/2;
   m_y=y1+MathAbs((y2-y1))/2;
//---      
   ArrayResize(m_weights,m_dimension);
//---    ;
   for(int i=0; i<m_dimension; i++) {m_weights[i]=min_values[i]+(max_values[i]-min_values[i])*rand()/32768;}
  };
//+------------------------------------------------------------------+
//|                                   |
//+------------------------------------------------------------------+
void CSOMNode::GetCoordinates(int &x1,int &y1,int &x2,int &y2)
  {
   x1=m_x1;
   y1=m_y1;
   x2=m_x2;
   y2=m_y2;
  }
//+------------------------------------------------------------------+
//|     weight_index                    |
//+------------------------------------------------------------------+
double CSOMNode::GetWeight(int weight_index)
  {
   if(weight_index>=0 && weight_index<m_dimension)
     {
      return(m_weights[weight_index]);
     }
   else return(0);
  }
//+------------------------------------------------------------------+
//|              |
//+------------------------------------------------------------------+
double CSOMNode::CalculateDistance(const double &vector[])
  {
   double dist_sqr=0;
   for(int i=0; i<m_dimension; i++)
     {
      dist_sqr+=(vector[i]-m_weights[i])*(vector[i]-m_weights[i]);
     }
   return(dist_sqr);
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double CSOMNode::CalculateDistance(const double &vector[],int dimension)
  {
   double dist_sqr=0; int dim=dimension<=m_dimension?dimension:m_dimension;
   for(int i=0; i<dim; i++)
     {
      dist_sqr+=(vector[i]-m_weights[i])*(vector[i]-m_weights[i]);
     }
   return(dist_sqr);
  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
double CSOMNode::CalculateDistance(const double &vector[],const bool &byte_map[])
  {
   double dist_sqr=0;
   for(int i=0; i<m_dimension; i++)
     {
      if(byte_map[i])
         dist_sqr+=(vector[i]-m_weights[i])*(vector[i]-m_weights[i]);
     }
   return(dist_sqr);
  };
//+------------------------------------------------------------------+
//|                |
//+------------------------------------------------------------------+
void CSOMNode::AdjustWeights(double &vector[],double learning_rate,double influence)
  {
   for(int i=0; i<m_dimension; i++)
     {
      m_weights[i]=m_weights[i]+learning_rate*influence*(vector[i]-m_weights[i]);
     }
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CSOMNode::SetDimension(int dimension)
  {
   m_dimension=dimension;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
int CSOMNode::GetDimension()
  {
   return(m_dimension);
  }
//+------------------------------------------------------------------+
